// Maomi.cpp : Defines the class behaviors for the application.
//

#include "stdafx.h"
#include "Maomi.h"
#include "MaomiConfig.h"
#include "MaomiDlg.h"

#include <process.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#endif


// CMaomiApp

BEGIN_MESSAGE_MAP(CMaomiApp, CWinApp)
	ON_COMMAND(ID_HELP, CWinApp::OnHelp)
END_MESSAGE_MAP()


// CMaomiApp construction

CMaomiApp::CMaomiApp()
: mLowLevelMouseHookDllH(NULL)
, mLowLevelMouseHookProc(NULL)
, mLowLevelMouseHookH(NULL)
, mThreadH(NULL)
, mShuttingDown(false)
, mMutexH(NULL)
, mConfigP(NULL)
{
	// TODO: add construction code here,
	// Place all significant initialization in InitInstance
}


// The one and only CMaomiApp objetc

CMaomiApp theApp;


// CMaomiApp initialization

BOOL CMaomiApp::InitInstance()
{
	mMutexH = CreateMutex(NULL, TRUE, "MamoiMutex");
	DWORD lastError = GetLastError();
	if (mMutexH != NULL && lastError == ERROR_ALREADY_EXISTS) {
		AfxMessageBox(_T("Another Instance is already running."), MB_ICONERROR);
		return FALSE;
	}

	// InitCommonControls() is required on Windows XP if an application
	// manifest specifies use of ComCtl32.dll version 6 or later to enable
	// visual styles.  Otherwise, any window creation will fail.
	InitCommonControls();

	CWinApp::InitInstance();

	AfxEnableControlContainer();

	// Standard initialization
	SetRegistryKey(_T("HWT"));
	mConfigP = new CMaomiConfig();
	mConfigP->Load();

	CMaomiDlg* dlgP = new CMaomiDlg(mConfigP);
	dlgP->Create(CMaomiDlg::IDD);

	m_pMainWnd = dlgP;

	// Install mouse hook after we loaded the UI.
	return InstallLowLevelMouseHook();
}

int CMaomiApp::ExitInstance()
{
	mShuttingDown = true;
	if (mThreadH != NULL) WaitForSingleObject(mThreadH, INFINITE);

	if (mLowLevelMouseHookH != NULL) {
		UnhookWindowsHookEx(mLowLevelMouseHookH);
	}
	if (mLowLevelMouseHookDllH != NULL) {
		FreeLibrary(mLowLevelMouseHookDllH);
	}

	if (m_pMainWnd != NULL) {
		delete m_pMainWnd;
	}

	if (mConfigP != NULL) {
		mConfigP->Save();
		delete mConfigP;
	}

	if (mMutexH != NULL) {
		CloseHandle(mMutexH);
	}
	return CWinApp::ExitInstance();
}

bool CMaomiApp::InstallLowLevelMouseHook()
{
	mLowLevelMouseHookDllH = LoadLibrary("Maomi.dll");
	if (mLowLevelMouseHookDllH == NULL) {
		AfxMessageBox(_T("Cannot load Mouse Hook DLL"), MB_ICONERROR);
		return false;
	}

	mLowLevelMouseHookProc = reinterpret_cast<HOOKPROC>(GetProcAddress(mLowLevelMouseHookDllH, "LowLevelMouseHookProc"));
	mLatestMsllHookStructP = reinterpret_cast<LPMSLLHOOKSTRUCT>(GetProcAddress(mLowLevelMouseHookDllH, "LatestMsllHookStruct"));
	mLatestTickP = reinterpret_cast<LPLONG>(GetProcAddress(mLowLevelMouseHookDllH, "LatestTick"));

	if (mLowLevelMouseHookProc == NULL || mLatestTickP == NULL || mLatestMsllHookStructP == NULL) {
		AfxMessageBox(_T("Invalid Mouse Hook DLL."), MB_ICONERROR);
		return false;
	}	

	mLowLevelMouseHookH = SetWindowsHookEx(WH_MOUSE_LL, mLowLevelMouseHookProc, mLowLevelMouseHookDllH, 0);
	if (mLowLevelMouseHookH == NULL) {
		AfxMessageBox(_T("Failed to install Mouse Hook."), MB_ICONERROR);
		return false;
	}

	mThreadH = reinterpret_cast<HANDLE>(_beginthread(MouseMonitorProc, 0, this));
	return mThreadH != NULL;
}

void CMaomiApp::MouseMonitorProc(void* aUserDataP)
{
	CMaomiApp* appP = reinterpret_cast<CMaomiApp*>(aUserDataP);
	CMaomiConfig* configP = appP->mConfigP;

	LONG  thresholdTick = 1000;	// 1 sec
	LONG  lastTick;
	LONG  currentTick = GetTickCount();
	LONG  lastTriggeredTick = currentTick;
	HWND  progmanHwnd = FindWindow(_T("Progman"), NULL);

	CRect rect;

	TCHAR bufferS[COMMAND_LEN];
	TCHAR comspecS[1024];

	GetClientRect(GetDesktopWindow(), &rect);
	GetEnvironmentVariable("COMSPEC", comspecS, sizeof(comspecS)/sizeof(TCHAR)-1);

	int Xs[NUM_CONFIG] = { 
		rect.left, (rect.left+rect.right)/2, rect.right, 
		rect.left, (rect.left+rect.right)/2, rect.right, 
		rect.left, (rect.left+rect.right)/2, rect.right 
	};
	int Ys[NUM_CONFIG] = {
		rect.top, rect.top, rect.top, 
		(rect.top+rect.bottom)/2, (rect.top+rect.bottom)/2, (rect.top+rect.bottom)/2, 
		rect.bottom, rect.bottom, rect.bottom
	};

	while (!appP->mShuttingDown) {
		currentTick = GetTickCount();
		lastTick = 	*(appP->mLatestTickP);
		if (lastTriggeredTick < lastTick) {
			int x = appP->mLatestMsllHookStructP->pt.x;
			int y = appP->mLatestMsllHookStructP->pt.y;

			for (int n = 0; n < NUM_CONFIG; ++n) {
				if (currentTick - lastTick > configP->mData[n].mDelay*thresholdTick) {
					HWND hwnd = GetForegroundWindow();
					if (configP->mData[n].mAction == 0) continue;
					if (configP->mData[n].mDesktopOnly &&  hwnd != progmanHwnd) continue;
					if (x < Xs[n] - configP->mData[n].mRadius || x > Xs[n] + configP->mData[n].mRadius) continue;
					if (y < Ys[n] - configP->mData[n].mRadius || y > Ys[n] + configP->mData[n].mRadius) continue;

					switch(configP->mData[n].mAction) {
						case 1:
							// Show Screen Saver
							SendMessage(GetDesktopWindow(), WM_SYSCOMMAND, SC_SCREENSAVE, 0);
							break;
						case 2: 
						{
							// Run Application
							_tcscpy(bufferS, configP->mData[n].mCommandS);
							LPTSTR fileNameS = bufferS;
							LPTSTR paramS = NULL;
							while (fileNameS[0] != '\0' && _istspace(fileNameS[0])) ++fileNameS;
							if (fileNameS[0] == '\"') {
								++fileNameS;
								paramS = _tcschr(fileNameS, '\"');
								if (paramS != NULL) paramS[0] = '\0';
							}
							else {
								paramS = _tcschr(fileNameS, ' ');
								if (paramS != NULL) {
									paramS[0] = '\0';
									++paramS;
								}
							}

							ShellExecute(appP->m_pMainWnd->GetSafeHwnd(), "open", fileNameS, paramS, NULL, SW_SHOWNORMAL);
							break;
						}
						default:
							break;
					}

					lastTriggeredTick = lastTick;
				}
			}
		}

		Sleep(thresholdTick);
	}
}
